(* module Lolevel *)

(* --------------------------------------------------------------- *)

module type PRINTING = sig
  val print : string -> unit
  val print_int : int -> unit

  val obox0 : unit -> unit
  val obox : unit -> unit
  val cbox : unit -> unit
  val break : unit -> unit
  val space : unit -> unit
  val flush : unit -> unit
  val newline : unit -> unit
  val force_newline : unit -> unit
end

module Printing = struct

  let print = Format.print_string

  let print_int = Format.print_int

  (* The printing functions call these utility functions to insert
     grouping information and line-breaking hints for the pretty-
     printing library:
       obox   Open a "box" whose contents will be indented by two
              spaces if the whole box cannot fit on the current line
       obox0  Same but indent continuation lines to the same column
              as the beginning of the box rather than 2 more columns
              to the right
       cbox   Close the current box
       break  Insert a breakpoint indicating where the line may be
              broken if necessary.
     See the documentation for the Format module in the OCaml library
     for more details.
  *)

  let obox0() = Format.open_hvbox 0
  let obox() = Format.open_hvbox 2
  let cbox = Format.close_box
  let break = Format.print_cut
  let space = Format.print_space
  let flush = Format.print_flush
  let newline = Format.print_newline (* flushes then prints "\n" *)
  let force_newline = Format.force_newline

end (* module Printing *)

(* --------------------------------------------------------------- *)

module Error = struct

  open Printing

  exception Exit of int

  type info = Lexing.position

  let void_fi = Lexing.dummy_pos

  type 'a withinfo = {i: info; v: 'a}

  type printer = unit -> unit

  let die pr = begin
    flush();
    Format.set_formatter_out_channel stderr;
    Format.open_vbox 0;
    obox0();
    pr();
    break();
    cbox();
    force_newline();
    raise (Exit 1)
  end

  let print_info (pos : info) : unit =
    (* In the text of Pierce's book, file positions in error messages
       are replaced with the string "Error:" *)
    if pos = void_fi
    then print "<Unknown file and line>: "
    else let fname = pos.Lexing.pos_fname in
      let lnum = pos.Lexing.pos_lnum in
      let cnum = pos.Lexing.pos_cnum - pos.Lexing.pos_bol in
      begin
        if fname = ""
        then print "<Unknown file>"
        else print fname;
        print ":";
        print_int lnum;
        print ".";
        print_int cnum;
        print ": "
      end

  let error s = die (fun()-> begin
    newline();
    print "Error: ";
    print s
  end)

  let error_with fi s = die (fun()-> begin
    newline();
    print_info fi;
    (* space(); *)
    print s
  end)

  (* Warnings print to stdout. *)

  let warning s = begin
    print "Warning: ";
    print s;
    newline()
  end

  let warning_with fi s = begin
    print_info fi;
    (* space(); *)
    print "Warning: ";
    print s;
    newline()
  end

end (* module Error *)
